#!/bin/bash
# EX.2020/11/26
# cpanel Fix Account Permissions
# https://github.com/PeachFlame/cPanel-fixperms
# https://www.thecpaneladmin.com/fix-account-permissions/
# 帳號備份檔，請放置 /backup/cpbackup/daily/account 目錄
# 執行此script之前，請先執行 find_sort_accounts.sh 取得帳號列表
# 請於 /backup/cpbackup/daily/account 目錄執行此 script

# 如果需要統計還原時間，可以取消以下註解，以及底下log相關註解
#rm -f /root/all_time.txt
#rm -f /root/restore_log
#date >> /root/all_time.txt

# 存放發現帳號home目錄存在，沒有執行還原帳號清單
touch /backup/cpbackup/daily/big_account_list_fail

# 將目前帳號列表複製一份出來 (需要比對時可以用的到)
/bin/cp -f /backup/cpbackup/daily/big_account_list /backup/cpbackup/daily/big_account_list_bak

filename="/backup/cpbackup/daily/big_account_list"
exec < $filename

while read line
do

# 如果還原時發現此帳號home目錄存在，則不還原，避免工程師手動還原，過一陣子因為此程式，又再一次還原
if [ -e /home/$line ]; then

echo "此帳號 $line 家目錄存在，不進行還原" >> /backup/cpbackup/daily/big_account_list_fail

else

# 帳號名稱
#echo "$line" >> /root/restore_log

# 將帳號public_html先移到/backup
mv /backup/cpbackup/daily/account/"$line"/homedir/public_html /backup

# 單一帳號還原 Start
#date >> /root/restore_log

# 執行帳號還原
cd /backup/cpbackup/daily/account/
/scripts/restorepkg --force --allow_reseller "$line"

# 單一帳號還原 End
#date >> /root/restore_log

# 使用rsync同步public_html
rsync -rvlHpDtP /backup/public_html/ /home/"$line"/public_html/

# 單一帳號rsync End
#date >> /root/restore_log

# 修正權與檔案權限
find /home/"$line"/public_html -type d -exec chmod 755 {} \;
find /home/"$line"/public_html -type f | xargs -d$'\n' -r chmod 644
find /home/"$line"/public_html -name '*.cgi' -o -name '*.pl' -o -name '*.perl' | xargs -r chmod 755

chown -R $line:$line /home/"$line"/public_html/
find /home/"$line"/public_html -name .htaccess -exec chown $line.$line {} \;
chown $line:nobody /home/"$line"/public_html

chmod 750 /home/"$line"/public_html

# 修正沒有放在 public_html 範圍內網站根目錄權限
for SUBDOMAIN in $(grep -i documentroot /var/cpanel/userdata/"$line"/* | grep -v '.cache\|_SSL' | awk '{print $2}' | grep -v public_html)
do
  find $SUBDOMAIN -type d -exec chmod 755 {} \;
  find $SUBDOMAIN -type f | xargs -d$'\n' -r chmod 644
  find $SUBDOMAIN -name '*.cgi' -o -name '*.pl' -o -name '*.perl' | xargs -r chmod 755

  chown -R $line:$line $SUBDOMAIN
  find $SUBDOMAIN -name .htaccess -exec chown $line.$line {} \;
  chmod 750 $SUBDOMAIN
  chown $line:nobody $SUBDOMAIN
done

# 修正放在 public_html 範圍內網站根目錄權限

for SUBDOMAIN in $(grep -i documentroot /var/cpanel/userdata/"$line"/* | grep -v '.cache\|_SSL' | awk '{print $2}' | grep public_html)
do
  chmod 750 $SUBDOMAIN
  chown $line:nobody $SUBDOMAIN
done

# 根據帳號備份檔案自動設定成原本IP，此為紀錄用，如果轉移IP不會更改，可取消註解
#cd /backup/cpbackup/daily/account/"$line"/cp
#IP=`cat "$line" | grep IP | awk -F\= '{print $2}'`
#/usr/local/cpanel/bin/setsiteip -u "$line" "$IP"

# 將帳號備份public_html放回原本位置
mv /backup/public_html /backup/cpbackup/daily/account/"$line"/homedir/

# 更新還原進度清單
sed -i "/$line/d" /backup/cpbackup/daily/big_account_list

#echo "" >> /root/restore_log
#echo "===============" >> /root/restore_log
#echo "" >> /root/restore_log

fi

# 加上檔案判斷，想要執行至一半中止，可使用新增 /tmp/stop 檔案
# 如果有先手動還原某個帳號，請手動更新 big_account_list 帳號列表，再一次手動執行此程式，即可繼續進行剩下的帳號還原作業
if [ -e /tmp/stop ]; then

echo ""
echo "目前還原至 $line 此帳號完成，程序已停止"
echo ""
rm -f /tmp/stop
exit

fi

done

# 如果需要統計還原時間，可以取消以下註解
#date >> /root/all_time.txt

